<?php

namespace Drupal\file_metadata_cleaner\Access;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Routing\Access\AccessInterface;
use Drupal\file\FileInterface;
use Drupal\file_metadata_cleaner\FileProcessor\FileProcessorPluginManager;
use Symfony\Component\Routing\Route;

/**
 * Access checker for the file metadata route.
 */
class FileMetadataCleanerAccessCheck implements AccessInterface {

  /**
   * The file processor plugin manager.
   *
   * @var \Drupal\file_metadata_cleaner\FileProcessor\FileProcessorPluginManager
   */
  protected FileProcessorPluginManager $pluginManager;

  /**
   * Constructs a new FileMetadataAccessCheck object.
   *
   * @param \Drupal\file_metadata_cleaner\FileProcessor\FileProcessorPluginManager $plugin_manager
   *   The file processor plugin manager.
   */
  public function __construct(FileProcessorPluginManager $plugin_manager) {
    $this->pluginManager = $plugin_manager;
  }

  /**
   * Checks access for the metadata route based on file MIME type.
   *
   * @param \Symfony\Component\Routing\Route $route
   *   The route being accessed.
   * @param \Drupal\file\FileInterface|null $file
   *   The loaded file entity from the route parameter.
   *
   * @return \Drupal\Core\Access\AccessResult
   *   The access result.
   */
  public function accessFile(Route $route, ?FileInterface $file = NULL): AccessResult {
    if (!$file) {
      return AccessResult::forbidden('File entity not found.');
    }

    $mime_type = $file->getMimeType();
    foreach ($this->pluginManager->getDefinitions() as $plugin_id => $definition) {
      $plugin = $this->pluginManager->createInstance($plugin_id);
      if ($plugin->supports($mime_type)) {
        return AccessResult::allowed()->addCacheableDependency($file);
      }
    }

    return AccessResult::forbidden("No plugin supports the file's MIME type.")->addCacheableDependency($file);
  }

  /**
   * Checks access for the metadata route based on plugin ID.
   */
  public function accessPlugin(Route $route, $plugin): AccessResult {
    if (!$this->pluginManager->hasDefinition($plugin)) {
      return AccessResult::forbidden("Plugin '$plugin' not found.");
    }

    return AccessResult::allowed();
  }

}
