<?php

declare(strict_types=1);

namespace Drupal\file_metadata_cleaner\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\file\FileInterface;
use Drupal\file_metadata_cleaner\Services\Exiftool;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Url;

/**
 * Controller for viewing metadata of files.
 */
class ReadMetadataController extends ControllerBase {

  use StringTranslationTrait;

  /**
   * The Exiftool service.
   *
   * @var \Drupal\file_metadata_cleaner\Services\Exiftool
   */
  protected Exiftool $exiftool;

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected FileSystemInterface $fileSystem;

  /**
   * Constructs the ReadMetadataController.
   *
   * @param \Drupal\file_metadata_cleaner\Services\Exiftool $exiftool
   *   The Exiftool service.
   * @param \Drupal\Core\File\FileSystemInterface $fileSystem
   *   The file system service.
   * @param \Drupal\Core\StringTranslation\TranslationInterface $string_translation
   *   The string translation service.
   */
  public function __construct(
    Exiftool $exiftool,
    FileSystemInterface $fileSystem,
    TranslationInterface $string_translation,
  ) {
    $this->exiftool = $exiftool;
    $this->fileSystem = $fileSystem;
    $this->stringTranslation = $string_translation;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    return new static(
      $container->get('file_metadata_cleaner.exiftool'),
      $container->get('file_system'),
      $container->get('string_translation'),
    );
  }

  /**
   * Displays metadata for the given file.
   *
   * @param \Drupal\file\FileInterface $file
   *   The file entity.
   *
   * @return array
   *   A render array containing file metadata and an action link.
   */
  public function view(FileInterface $file): array {
    $uri = $file->getFileUri();
    $path = $this->fileSystem->realpath($uri);
    $metadata = $this->exiftool->readMetadata($path);

    $rows = [];
    foreach ($metadata as $key => $value) {
      $rows[] = [
        htmlspecialchars((string) $key, ENT_QUOTES, 'UTF-8'),
        htmlspecialchars((string) $value, ENT_QUOTES, 'UTF-8'),
      ];
    }
    $cleaned = "File hasn't been cleaned by File Metadata Cleaner yet.";

    if ($file->get('metadata_cleaned')->value) {
      $cleaned = "File has already been cleaned by File Metadata Cleaner.";
    }
    return [
      'actions' => [
        '#type' => 'container',
        'clean' => [
          '#type' => 'link',
          '#title' => $this->t('Clean Metadata'),
          '#url' => Url::fromRoute('file_metadata_cleaner.clean_metadata', ['file' => $file->id()]),
          '#attributes' => ['class' => ['button', 'button--primary']],
        ],
        'status' => [
          '#markup' => '<p><b>' . $cleaned . '</b></p>',
        ],
      ],
      'metadata' => [
        '#type' => 'table',
        '#header' => [
          $this->t('Field'),
          $this->t('Value'),
        ],
        '#rows' => $rows,
      ],
    ];
  }

}
