<?php

namespace Drupal\file_metadata_cleaner\Form;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\Component\Plugin\PluginManagerInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a confirmation form for cleaning file metadata.
 */
class CleanMetadataConfirmForm extends ConfirmFormBase {

  /**
   * The file entity for which metadata will be cleaned.
   *
   * @var \Drupal\Core\Entity\EntityInterface|null
   */
  protected ?EntityInterface $fileEntity = NULL;

  /**
   * The file processor plugin manager.
   *
   * @var \Drupal\Component\Plugin\PluginManagerInterface
   */
  protected PluginManagerInterface $pluginManager;

  /**
   * The current route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * Constructs a new CleanMetadataConfirmForm object.
   *
   * @param \Drupal\Component\Plugin\PluginManagerInterface $plugin_manager
   *   The file processor plugin manager.
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The current route match.
   */
  public function __construct(PluginManagerInterface $plugin_manager, RouteMatchInterface $route_match) {
    $this->pluginManager = $plugin_manager;
    $this->routeMatch = $route_match;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    return new static(
      $container->get('plugin.manager.file_metadata_cleaner.processor'),
      $container->get('current_route_match')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'file_metadata_cleaner_clean_metadata_confirm_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $file = $this->routeMatch->getParameter('file');
    if ($file instanceof EntityInterface) {
      $this->fileEntity = $file;
    }
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion(): string {
    return $this->t('Are you sure you want to clean metadata for the file @filename?', [
      '@filename' => $this->fileEntity?->getFilename() ?? $this->t('unknown'),
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl(): Url {
    return Url::fromRoute('view.files.page_1');
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmText(): string {
    return $this->t('Yes, clean metadata');
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    foreach ($this->pluginManager->getDefinitions() as $plugin_id => $definition) {
      $instance = $this->pluginManager->createInstance($plugin_id);
      if ($instance->supports($this->fileEntity->getMimeType())) {
        $instance->process($this->fileEntity);
        break;
      }
    }

    $form_state->setRedirectUrl($this->getCancelUrl());
  }

}
