<?php

namespace Drupal\file_metadata_cleaner\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\file_metadata_cleaner\FileProcessor\FileProcessorPluginManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Routing\RouteMatchInterface;

/**
 * Form to edit file processor metadata cleaning settings.
 */
class FileProcessorSettingsForm extends ConfigFormBase {

  /**
   * The plugin manager service.
   *
   * @var \Drupal\file_metadata_cleaner\FileProcessor\FileProcessorPluginManager
   */
  protected FileProcessorPluginManager $pluginManager;

  /**
   * The plugin ID being configured.
   *
   * @var string|null
   */
  protected ?string $pluginId = NULL;

  /**
   * The current route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * Constructs the form.
   */
  public function __construct(FileProcessorPluginManager $pluginManager, RouteMatchInterface $routeMatch) {
    $this->pluginManager = $pluginManager;
    $this->routeMatch = $routeMatch;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    return new static(
      $container->get('plugin.manager.file_metadata_cleaner.processor'),
      $container->get('current_route_match'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'file_metadata_cleaner_processor_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $this->pluginId = $this->routeMatch->getParameter('plugin');
    $config = $this->config('file_metadata_cleaner.settings');
    $metadata = $config->get('metadata')[$this->pluginId] ?? [];

    if (empty($metadata)) {
      $form['description'] = [
        '#markup' => $this->t('Metadata config is empty for plugin : @plugin', ['@plugin' => $this->pluginId]),
      ];
      return parent::buildForm($form, $form_state);
    }

    $form['description'] = [
      '#markup' => $this->t('Configure metadata cleaning for processor: @plugin', ['@plugin' => $this->pluginId]),
    ];

    foreach ($metadata as $key => $value) {
      $form[$key] = [
        '#type' => 'textfield',
        '#title' => $key,
        '#default_value' => $value,
        '#description' => $this->t('Use -1 to preserve original metadata value, or provide a new value. Empty to clear metadata.'),
      ];
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    foreach (array_keys($form) as $key) {
      if (!is_array($form[$key]) || !isset($form[$key]['#type']) || $form[$key]['#type'] !== 'textfield') {
        continue;
      }

      $value = trim($form_state->getValue($key));

      if (strlen($value) > 255) {
        $form_state->setErrorByName($key, $this->t('The value for %key is too long. Maximum is 255 characters.', ['%key' => $key]));
      }

      if ($value !== strip_tags($value)) {
        $form_state->setErrorByName($key, $this->t('HTML tags are not allowed in %key.', ['%key' => $key]));
      }

      if ($value !== '' && $value !== '-1' && !preg_match('/^[\w\s\.\,\-\_]+$/u', $value)) {
        $form_state->setErrorByName($key, $this->t('The value for %key contains invalid characters.', ['%key' => $key]));
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $config = $this->config('file_metadata_cleaner.settings');
    $metadata = $config->get('metadata');

    $values = [];
    foreach (array_keys($metadata[$this->pluginId] ?? []) as $key) {
      $values[$key] = $form_state->getValue($key);
    }

    $metadata[$this->pluginId] = $values;

    $config->set('metadata', $metadata);
    $config->save();

    parent::submitForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['file_metadata_cleaner.settings'];
  }

}
