<?php

namespace Drupal\file_metadata_cleaner\Hooks;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Routing\RouteProviderInterface;
use Drupal\Core\Url;
use Drupal\file\FileInterface;
use Drupal\file_metadata_cleaner\Access\FileMetadataCleanerAccessCheck;
use Drupal\file_metadata_cleaner\FileProcessor\FileProcessorPluginManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;

/**
 * Handles file entity hooks for metadata cleaning.
 */
final class FileEntity implements ContainerInjectionInterface {

  /**
   * The plugin manager for file processors.
   *
   * @var \Drupal\file_metadata_cleaner\FileProcessor\FileProcessorPluginManager
   */
  protected FileProcessorPluginManager $pluginManager;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * The file metadata access checker.
   *
   * @var \Drupal\file_metadata_cleaner\Access\FileMetadataCleanerAccessCheck
   */
  protected FileMetadataCleanerAccessCheck $accessChecker;

  /**
   * The route provider service.
   *
   * @var \Drupal\Core\Routing\RouteProviderInterface
   */
  protected RouteProviderInterface $routeProvider;

  /**
   * Constructs a new FileEntity object.
   *
   * @param \Drupal\file_metadata_cleaner\FileProcessor\FileProcessorPluginManager $pluginManager
   *   The plugin manager.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory.
   * @param \Drupal\file_metadata_cleaner\Access\FileMetadataCleanerAccessCheck $accessChecker
   *   The file metadata access checker.
   * @param \Drupal\Core\Routing\RouteProviderInterface $routeProvider
   *   The route provider service.
   */
  public function __construct(
    FileProcessorPluginManager $pluginManager,
    ConfigFactoryInterface $configFactory,
    FileMetadataCleanerAccessCheck $accessChecker,
    RouteProviderInterface $routeProvider,
  ) {
    $this->pluginManager = $pluginManager;
    $this->configFactory = $configFactory;
    $this->accessChecker = $accessChecker;
    $this->routeProvider = $routeProvider;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    return new static(
      $container->get('plugin.manager.file_metadata_cleaner.processor'),
      $container->get('config.factory'),
      $container->get('file_metadata_cleaner.access_check'),
      $container->get('router.route_provider'),
    );
  }

  /**
   * Hook implementation for file insert.
   *
   * @param \Drupal\file\FileInterface $file
   *   The file entity being inserted.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   *   Throws PluginException.
   */
  public function fileInsert(FileInterface $file): void {
    $config = $this->configFactory->get('file_metadata_cleaner.settings');

    if (!$config->get('clean_on_upload')) {
      return;
    }

    foreach ($this->pluginManager->getDefinitions() as $plugin_id => $definition) {
      $instance = $this->pluginManager->createInstance($plugin_id);

      if ($instance->supports($file->getMimeType())) {
        $instance->process($file);
        break;
      }
    }
  }

  /**
   * Alters entity operations for file entities to add a Metadata button.
   *
   * @param array $operations
   *   The list of entity operations.
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity being altered.
   */
  public function operationAlter(array &$operations, EntityInterface $entity): void {
    if ($entity->getEntityTypeId() === 'file') {
      $route = $this->routeProvider->getRouteByName('file_metadata_cleaner.read_metadata');
      $access = $this->accessChecker->accessFile($route, $entity);
      if ($access->isAllowed()) {
        $operations['metadata'] = [
          'title' => 'Metadata',
          'weight' => 94,
          'url' => Url::fromRoute('file_metadata_cleaner.read_metadata', [
            'file' => $entity->id(),
          ]),
          'attributes' => [
            'class' => ['use-ajax'],
            'data-dialog-type' => 'modal',
            'data-dialog-options' => json_encode([
              'width' => 800,
            ]),
          ],
        ];
      }
    }
  }

}
