<?php

namespace Drupal\file_metadata_cleaner\Plugin\FileProcessor;

use Drupal\Component\Utility\Html;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Plugin\PluginBase;
use Drupal\file\FileInterface;
use Drupal\file_metadata_cleaner\Services\Exiftool;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Base class for File Processor plugins.
 */
abstract class FileProcessorBase extends PluginBase implements FileProcessorInterface, ContainerFactoryPluginInterface {

  /**
   * The Exiftool service.
   *
   * @var \Drupal\file_metadata_cleaner\Services\Exiftool
   */
  protected Exiftool $exiftool;

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected FileSystemInterface $fileSystem;

  /**
   * The logger service.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected LoggerChannelInterface $logger;

  /**
   * The metadata configuration array.
   *
   * @var array
   */
  protected array $config;

  /**
   * Constructs a FileProcessorBase plugin.
   *
   * @param array $configuration
   *   The plugin configuration array.
   * @param string $plugin_id
   *   The plugin ID.
   * @param mixed $plugin_definition
   *   The plugin definition.
   * @param \Drupal\file_metadata_cleaner\Services\Exiftool $exiftool
   *   The Exiftool service.
   * @param \Drupal\Core\File\FileSystemInterface $fileSystem
   *   The file system service.
   * @param \Drupal\Core\Logger\LoggerChannelInterface $logger
   *   The logger service.
   * @param array $config
   *   The metadata configuration.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    Exiftool $exiftool,
    FileSystemInterface $fileSystem,
    LoggerChannelInterface $logger,
    array $config,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->exiftool = $exiftool;
    $this->fileSystem = $fileSystem;
    $this->logger = $logger;
    $this->config = $config;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): static {
    $settings = $container->get('config.factory')->get('file_metadata_cleaner.settings');
    $metadata = $settings->get('metadata') ?? [];

    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('file_metadata_cleaner.exiftool'),
      $container->get('file_system'),
      $container->get('logger.channel.file_metadata_cleaner'),
      $metadata
    );
  }

  /**
   * {@inheritdoc}
   */
  public function supports(string $mimeType): bool {
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function supportedMetadataWithValues(): array {
    return $this->config[$this->getPluginId()];
  }

  /**
   * {@inheritdoc}
   */
  public function process(FileInterface $file): void {
    $path = $this->fileSystem->realpath($file->getFileUri());
    $metadata = $this->exiftool->readMetadata($path);

    $args = [];
    foreach ($this->supportedMetadataWithValues() as $key => $value) {
      if ($value === '-1' && array_key_exists($key, $metadata)) {
        $args[] = "-{$key}=" . addslashes($metadata[$key]);
      }
      elseif ($value !== '-1' && trim($value) !== '') {
        $args[] = "-{$key}=" . addslashes($value);
      }
    }

    $output = $this->exiftool->writeMetadata($args, $path);
    $this->logger->notice('Metadata cleaned for file @path: @output', [
      '@path' => $path,
      '@output' => Html::escape($output),
    ]);

    if ($file->hasField('metadata_cleaned')) {
      $file->set('metadata_cleaned', TRUE);
    }
    $file->save();
  }

}
