<?php

namespace Drupal\file_metadata_cleaner\Services;

use Drupal\Core\DrupalKernelInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\Process\Exception\ProcessFailedException;
use Symfony\Component\Process\Process;

/**
 * Provides a metadata processor implementation using ExifTool.
 */
class Exiftool implements MetadataProcessorInterface {

  /**
   * The path to the ExifTool binary.
   *
   * @var string
   */
  protected string $exiftoolPath;

  /**
   * The logger service.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected LoggerInterface $logger;

  /**
   * Constructs a new Exiftool service.
   *
   * @param \Drupal\Core\DrupalKernelInterface $kernel
   *   The Drupal kernel service.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger service.
   */
  public function __construct(DrupalKernelInterface $kernel, LoggerInterface $logger) {
    $this->exiftoolPath = $kernel->getAppRoot() . '/../vendor/ahmetburkan/exiftool-binary/exiftool';
    $this->logger = $logger;
  }

  /**
   * {@inheritdoc}
   */
  public function readMetadata(string $filePath): array {
    $safePath = $this->validateFilePath($filePath);

    $process = new Process([$this->exiftoolPath, $safePath]);
    $process->setTimeout(10);
    $process->run();

    if (!$process->isSuccessful()) {
      $this->logger->error('ExifTool command failed: @error', [
        '@error' => $process->getErrorOutput(),
      ]);
      throw new ProcessFailedException($process);
    }

    return $this->formatMetadata($process->getOutput());
  }

  /**
   * {@inheritdoc}
   */
  public function writeMetadata(array $arguments, string $filePath): string {
    $this->validateArguments($arguments);
    $safePath = $this->validateFilePath($filePath);

    if (!is_writable($safePath)) {
      throw new \InvalidArgumentException("File is not writable: {$safePath}");
    }

    $command = array_merge(
      [$this->exiftoolPath, '-overwrite_original', '-all='],
      $arguments,
      [$safePath]
    );

    $process = new Process($command);
    $process->setTimeout(10);
    $process->run();

    if (!$process->isSuccessful()) {
      $this->logger->error('ExifTool command failed: @error', [
        '@error' => $process->getErrorOutput(),
      ]);
      throw new ProcessFailedException($process);
    }

    return $process->getOutput();
  }

  /**
   * Converts ExifTool output into a key-value array.
   *
   * @param string $metadataFromFile
   *   The raw metadata string.
   *
   * @return array
   *   Formatted metadata.
   */
  public function formatMetadata(string $metadataFromFile): array {
    $formattedMetadataFromFile = [];
    foreach (explode("\n", $metadataFromFile) as $line) {
      if (str_contains($line, ': ')) {
        [$key, $value] = explode(': ', $line, 2);
        $formattedMetadataFromFile[trim($key)] = trim($value);
      }
    }
    return $formattedMetadataFromFile;
  }

  /**
   * Validates metadata arguments to prevent unsafe command input.
   *
   * @param array $arguments
   *   Metadata arguments.
   *
   * @throws \InvalidArgumentException
   *   If arguments are invalid.
   */
  protected function validateArguments(array $arguments): void {
    foreach ($arguments as $arg) {
      if (!is_string($arg)) {
        throw new \InvalidArgumentException('All arguments must be strings.');
      }
      if (preg_match('/[;&|`$<>]/u', $arg)) {
        throw new \InvalidArgumentException("Unsafe characters detected in argument: {$arg}");
      }
    }
  }

  /**
   * Validates a file path.
   *
   * @param string $filePath
   *   The file path.
   *
   * @return string
   *   Validated file path.
   *
   * @throws \InvalidArgumentException
   *   If the path is invalid or unsafe.
   */
  protected function validateFilePath(string $filePath): string {
    $blacklistPatterns = [
      '/\.\./',
      '/[;&|`$<>]/u',
      '/[\x00-\x1F\x7F]/u',
      '/[\xC0-\xC1]/u',
      '/[\x{202E}]/u',
    ];

    foreach ($blacklistPatterns as $pattern) {
      if (preg_match($pattern, $filePath)) {
        throw new \InvalidArgumentException("Unsafe file path detected: {$filePath}");
      }
    }

    if (strlen($filePath) > 4096) {
      throw new \InvalidArgumentException("File path too long: {$filePath}");
    }

    if (!file_exists($filePath)) {
      throw new \InvalidArgumentException("File does not exist: {$filePath}");
    }

    $resolvedPath = realpath($filePath);
    if (!$resolvedPath || !is_readable($resolvedPath)) {
      throw new \InvalidArgumentException("Invalid or unreadable file: {$resolvedPath}");
    }

    return $resolvedPath;
  }

}
