<?php

namespace Drupal\Tests\file_metadata_cleaner\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Drupal\file_metadata_cleaner\Services\Exiftool;
use Drupal\Core\File\FileSystemInterface;

/**
 * Tests the Exiftool service interaction.
 *
 * @group file_metadata_cleaner
 */
class ExiftoolTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['file_metadata_cleaner', 'system'];

  /**
   * The Exiftool service.
   *
   * @var \Drupal\file_metadata_cleaner\Services\Exiftool
   */
  protected Exiftool $exiftool;

  /**
   * The filesystem service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected FileSystemInterface $fileSystem;

  /**
   * Path to the temporary test image.
   *
   * @var string
   */
  protected string $filePath;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->installConfig(['file_metadata_cleaner']);
    $this->exiftool = $this->container->get('file_metadata_cleaner.exiftool');
    $this->fileSystem = $this->container->get('file_system');
    $this->filePath = sys_get_temp_dir() . '/test_file.jpeg';

    $jpegData = base64_decode('/9j/4AAQSkZJRgABAQEAYABgAAD/2wBDAAMCAgICAgMCAgIDAwMDBAYEBAQEBQYFBQUFBggGBgYGBgcICQsJCAgKCAcHCg0KCgsMDAwMDAwMDAwMDAz/wAALCAA
BAAEBAREA/8QAFQABAQAAAAAAAAAAAAAAAAAAAAf/xAAUEAEAAAAAAAAAAAAAAAAAAAAA/9oADAMBAAIQAxAAAAGAf//Z');
    file_put_contents($this->filePath, $jpegData);
  }

  /**
   * {@inheritdoc}
   */
  protected function tearDown(): void {
    if ($this->fileSystem->realpath($this->filePath)) {
      $this->fileSystem->delete($this->filePath);
    }
    parent::tearDown();
  }

  /**
   * Tests that unsafe characters in arguments throw an exception.
   *
   * @param string $argument
   *   A potentially malicious metadata argument.
   *
   * @dataProvider maliciousArgumentsProvider
   */
  public function testWriteExiftoolUnsafeArguments(string $argument): void {
    $this->expectException(\InvalidArgumentException::class);
    $this->exiftool->writeMetadata([$argument], $this->filePath);
  }

  /**
   * Data provider for malicious metadata argument injection attempts.
   *
   * @return array
   *   List of dangerous argument examples.
   */
  public static function maliciousArgumentsProvider(): array {
    return [
      ['-Title=hello; rm -rf /'],
      ['-Title=hello && shutdown -h now'],
      ['-Title=hello | ls -la'],
      ['-Title=hello `whoami`'],
      ['-Title=hello $(uname -a)'],
      ['-Title=hello > /tmp/pwned'],
      ['-Title=hello < /etc/passwd'],
      ['-Title=hello & echo hacked'],
      ['-Title=hello || echo pwned'],
      ['-Title=foo`echo bar`'],
    ];
  }

  /**
   * Tests that unsafe file paths throw an exception.
   *
   * @param string $path
   *   A potentially malicious file path.
   *
   * @dataProvider maliciousFilePathsProvider
   */
  public function testWriteExiftoolUnsafeFilePaths(string $path): void {
    $this->expectException(\InvalidArgumentException::class);
    $this->exiftool->writeMetadata(['-Author=Safe'], $path);
  }

  /**
   * Data provider for malicious file path injection attempts.
   *
   * @return array
   *   List of dangerous file path examples.
   */
  public static function maliciousFilePathsProvider(): array {
    return [
      ['/tmp/test;rm -rf /.jpeg'],
      ['/tmp/test && whoami.jpeg'],
      ['/tmp/test | ls.jpeg'],
      ['/tmp/test.jpeg;echo hacked'],
      ['/tmp/test.jpeg > /tmp/hacked'],
      ['/tmp/../etc/passwd'],
      ['/tmp/test`rm -rf /`.jpeg'],
      ['/tmp/$(whoami).jpeg'],
      ['/tmp/`ls`.jpeg'],
    ];
  }

  /**
   * Tests advanced edge-case file paths.
   *
   * @param string $path
   *   A potentially malformed file path.
   *
   * @dataProvider edgeCaseFilePathsProvider
   */
  public function testWriteExiftoolEdgeCaseFilePaths(string $path): void {
    $this->expectException(\InvalidArgumentException::class);
    $this->exiftool->writeMetadata(['-Author=Safe'], $path);
  }

  /**
   * Provides edge-case file paths with special or control characters.
   *
   * @return array
   *   List of edge-case paths.
   */
  public static function edgeCaseFilePathsProvider(): array {
    return [
      ["/tmp/legitfile.jpeg\x00.png"],
      ["/tmp/\x1B[31mfile.jpeg"],
      ["/tmp/\xC0\xAEjpeg"],
      ["/tmp/👻\x00rm.jpeg"],
      ["/tmp/" . str_repeat("a", 4096) . ".jpeg"],
      ["/tmp/\n\r\t.jpeg"],
      ["/tmp/\u{202E}gpj.elif"],
      ["\u{0000}/tmp/injection.jpeg"],
    ];
  }

  /**
   * Tests successful metadata reading from a safe file.
   */
  public function testReadMetadataSuccess(): void {
    $output = $this->exiftool->readMetadata($this->filePath);
    $this->assertIsArray($output);
    $this->assertStringContainsString('JPEG', $output['File Type']);
  }

}
